-- PixelSnake
-- Alex Page  2011-08-07

segmentsize = 4

function toWindowCoord(input)
	return input*segmentsize
end

-- BEGIN SNAKE CLASS --
Snake = class()

function Snake:init()
	self.body = {}
	self.length = 10
	self.c = 10
	self.r = 10
	self.dir = {0, 1}
	self.crashed = false
	self.eat = false
	self.allowturn = true
end

function Snake:changeDirection(key)
	UP = {0, -1}
	DOWN = {0, 1}
	LEFT = {-1, 0}
	RIGHT = {1, 0}
	
	if key == '4' and self.dir[1] <= 0 and self.allowturn then
		self.dir = LEFT
		self.allowturn = false
	elseif key == '6' and self.dir[1] >= 0 and self.allowturn then
		self.dir = RIGHT
		self.allowturn = false
	elseif key == '8' and self.dir[2] <= 0 and self.allowturn then
		self.dir = UP
		self.allowturn = false
	elseif key == '5' and self.dir[2] >= 0 and self.allowturn then
		self.dir = DOWN
		self.allowturn = false
	end
end

function Snake:move(food)
	self.allowturn = true
	
	self.eat = false
	
	self.c = self.c + self.dir[1]
	self.r = self.r + self.dir[2]
	
	for i,v in ipairs(self.body) do
		if self.c == v[1] and self.r == v[2] then
			self.crashed = true
		end
	end
	
	if toWindowCoord(self.c) >= platform.window:width() - segmentsize or self.c <= 0 or toWindowCoord(self.r) >= platform.window:height() or self.r <= 5 then
		self.crashed = true
	end
	
	if self.c == food.c and self.r == food.r then
		self.eat = true
		self.length = self.length + 10
	end
	
	if (# self.body) <= self.length then
		table.insert(self.body, 1, {self.c, self.r})
	end
	
	if (# self.body) > self.length then
		table.remove(self.body)
	end
end

function Snake:draw(gc)
	for i,v in ipairs(self.body) do
		if i == 1 then
			gc:setColorRGB(255, 0, 0)
		else
			gc:setColorRGB(255, 255, 255)
		end
		gc:fillRect(toWindowCoord(v[1]), toWindowCoord(v[2]), segmentsize, segmentsize)
	end
end
-- END SNAKE CLASS --

-- BEGIN FOOD CLASS --
Food = class()

function Food:init()
	clear = false
	while not clear do
		self.c = math.random((platform.window:width() / segmentsize) - 1)
		self.r = math.random(6, (platform.window:height() / segmentsize) - 1)
		clear = true
		for i,v in ipairs(snake.body) do
			if self.c == v[1] and self.r == v[2] then
				clear = false
			end
		end
	end
end

function Food:draw(gc)
	gc:setColorRGB(0, 255, 0)
	gc:fillRect(toWindowCoord(self.c), toWindowCoord(self.r), segmentsize, segmentsize)
end
-- END FOOD CLASS --

function drawUI(gc)
	gc:setColorRGB(255, 255, 255)
	gc:drawLine(0, 19, platform.window:width(), 19)
	
	 gc:setFont("sansserif", "r", 9)
	 
	 gc:drawString("Score: " .. score, 4, 15)
	 
	 if not paused and not snake.crashed then
	 	statusString = "Press 9 to Pause"
	 elseif paused and not snake.crashed then
	 	statusString = "Press 9 to Unpause or 7 to Restart"
	 elseif snake.crashed then
	 	statusString = "Game Over! Press 7 to Restart"
	 end
	 
	 gc:drawString(statusString, platform.window:width() - gc:getStringWidth(statusString) - 4, 15)
	 
	 
end

function on.create()
    paused = true
    snake = Snake()
    food = Food()
    score = 0
end

function on.resize(w, h)
	on.create()
end

function on.paint(gc)
	gc:setColorRGB(0, 0, 0)
	gc:fillRect(0, 0, platform.window:width(), platform.window:height())
	
	snake:draw(gc)
	food:draw(gc)
	drawUI(gc)
end

function on.timer()
	if not snake.crashed then
		snake:move(food)
	else
		timer.stop()
		paused = true
    end
	if snake.eat then
		food = Food()
		score = score + 1
	end
	platform.window:invalidate()
end

function on.charIn(ch)
    if ch == '7' and paused then -- restart game
    	on.create()
    	paused = false
    	timer.start(0.1)
    elseif ch == '9' then   -- pause/play
        if not paused then
            timer.stop()
            paused = true
        else
            paused = false
            timer.start(0.1)
        end
    else
    	snake:changeDirection(ch)
    end
    platform.window:invalidate()
end
